@@ -147,12 +147,23 @@ void _mosquitto_check_keepalive(struct mosquitto *mosq)
 
 uint16_t _mosquitto_mid_generate(struct mosquitto *mosq)
 {
+	/* FIXME - this would be better with atomic increment, but this is safer
+	 * for now for a bug fix release.
+	 *
+	 * If this is changed to use atomic increment, callers of this function
+	 * will have to be aware that they may receive a 0 result, which may not be
+	 * used as a mid.
+	 */
+	uint16_t mid;
 	assert(mosq);
 
+	pthread_mutex_lock(&mosq->mid_mutex);
 	mosq->last_mid++;
 	if(mosq->last_mid == 0) mosq->last_mid++;
+	mid = mosq->last_mid;
+	pthread_mutex_unlock(&mosq->mid_mutex);
 	
-	return mosq->last_mid;
+	return mid;
 }
 
 /* Check that a topic used for publishing is valid.
@@ -307,10 +318,10 @@ int _mosquitto_hex2bin(const char *hex, unsigned char *bin, int bin_max_len)
 FILE *_mosquitto_fopen(const char *path, const char *mode)
 {
 #ifdef WIN32
-	char buf[MAX_PATH];
+	char buf[4096];
 	int rc;
-	rc = ExpandEnvironmentStrings(path, buf, MAX_PATH);
-	if(rc == 0 || rc == MAX_PATH){
+	rc = ExpandEnvironmentStrings(path, buf, 4096);
+	if(rc == 0 || rc > 4096){
 		return NULL;
 	}else{
 		return fopen(buf, mode);
