@@ -452,6 +452,7 @@ int mqtt3_config_read(struct mqtt3_config *config, bool reload)
 	int rc = MOSQ_ERR_SUCCESS;
 	struct config_recurse cr;
 	int lineno;
+	int len;
 #ifdef WITH_BRIDGE
 	int i;
 #endif
@@ -485,9 +486,10 @@ int mqtt3_config_read(struct mqtt3_config *config, bool reload)
 			_mosquitto_free(config->persistence_filepath);
 		}
 		if(config->persistence_location && strlen(config->persistence_location)){
-			config->persistence_filepath = _mosquitto_malloc(strlen(config->persistence_location) + strlen(config->persistence_file) + 1);
+			len = strlen(config->persistence_location) + strlen(config->persistence_file) + 1;
+			config->persistence_filepath = _mosquitto_malloc(len);
 			if(!config->persistence_filepath) return MOSQ_ERR_NOMEM;
-			sprintf(config->persistence_filepath, "%s%s", config->persistence_location, config->persistence_file);
+			snprintf(config->persistence_filepath, len, "%s%s", config->persistence_location, config->persistence_file);
 		}else{
 			config->persistence_filepath = _mosquitto_strdup(config->persistence_file);
 			if(!config->persistence_filepath) return MOSQ_ERR_NOMEM;
@@ -638,7 +640,7 @@ int _config_read_file_core(struct mqtt3_config *config, bool reload, const char
 					if(!key){
 						_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 						return MOSQ_ERR_NOMEM;
-					}else if(strlen(key) == 0){
+					}else if(STREMPTY(key)){
 						_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Invalid auth_opt_ config option.");
 						return MOSQ_ERR_INVAL;
 					}
@@ -1424,7 +1426,7 @@ int _config_read_file_core(struct mqtt3_config *config, bool reload, const char
 					}
 				}else if(!strcmp(token, "message_size_limit")){
 					if(_conf_parse_int(&token, "message_size_limit", (int *)&config->message_size_limit, saveptr)) return MOSQ_ERR_INVAL;
-					if(config->message_size_limit < 0 || config->message_size_limit > MQTT_MAX_PAYLOAD){
+					if(config->message_size_limit > MQTT_MAX_PAYLOAD){
 						_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Invalid message_size_limit value (%d).", config->message_size_limit);
 						return MOSQ_ERR_INVAL;
 					}
@@ -1865,9 +1867,6 @@ int _config_read_file_core(struct mqtt3_config *config, bool reload, const char
 				}else if(!strcmp(token, "use_username_as_clientid")){
 					if(reload) continue; // Listeners not valid for reloading.
 					if(_conf_parse_bool(&token, "use_username_as_clientid", &cur_listener->use_username_as_clientid, saveptr)) return MOSQ_ERR_INVAL;
-				}else if(!strcmp(token, "user")){
-					if(reload) continue; // Drop privileges user not valid for reloading.
-					if(_conf_parse_string(&token, "user", &config->user, saveptr)) return MOSQ_ERR_INVAL;
 				}else if(!strcmp(token, "username") || !strcmp(token, "remote_username")){
 #ifdef WITH_BRIDGE
 					if(reload) continue; // FIXME
