@@ -1,30 +1,17 @@
 /*
-Copyright (c) 2012,2013 Roger Light <roger@atchoo.org>
-All rights reserved.
+Copyright (c) 2012-2014 Roger Light <roger@atchoo.org>
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
-
-1. Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
-2. Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-3. Neither the name of mosquitto nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
-LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-POSSIBILITY OF SUCH DAMAGE.
+All rights reserved. This program and the accompanying materials
+are made available under the terms of the Eclipse Public License v1.0
+and Eclipse Distribution License v1.0 which accompany this distribution.
+ 
+The Eclipse Public License is available at
+   http://www.eclipse.org/legal/epl-v10.html
+and the Eclipse Distribution License is available at
+  http://www.eclipse.org/org/documents/edl-v10.php.
+ 
+Contributors:
+   Roger Light - initial implementation and documentation.
 */
 
 
@@ -69,7 +56,7 @@ int base64_encode(unsigned char *in, unsigned int in_len, char **encoded)
 		return 1;
 	}
 	BIO_get_mem_ptr(b64, &bptr);
-	*encoded = calloc(bptr->length+1, 1);
+	*encoded = malloc(bptr->length+1);
 	if(!(*encoded)){
 		BIO_free_all(b64);
 		return 1;
@@ -86,7 +73,9 @@ void print_usage(void)
 {
 	printf("mosquitto_passwd is a tool for managing password files for mosquitto.\n\n");
 	printf("Usage: mosquitto_passwd [-c | -D] passwordfile username\n");
+	printf("       mosquitto_passwd -b passwordfile username password\n");
 	printf("       mosquitto_passwd -U passwordfile\n");
+	printf(" -b : run in batch mode to allow passing passwords on the command line.\n");
 	printf(" -c : create a new password file. This will overwrite existing files.\n");
 	printf(" -D : delete the username rather than adding/updating its password.\n");
 	printf(" -U : update a plain text password file to use hashed passwords.\n");
@@ -119,6 +108,7 @@ int output_new_password(FILE *fptr, const char *username, const char *password)
 
 	digest = EVP_get_digestbyname("sha512");
 	if(!digest){
+		if(salt64) free(salt64);
 		fprintf(stderr, "Error: Unable to create openssl digest.\n");
 		return 1;
 	}
@@ -356,8 +346,11 @@ void handle_sigint(int signal)
 
 int main(int argc, char *argv[])
 {
-	char *password_file = NULL;
+	char *password_file_tmp = NULL;
+	char password_file[1024];
 	char *username = NULL;
+	char *password_cmd = NULL;
+	bool batch_mode = false;
 	bool create_new = false;
 	bool delete_user = false;
 	FILE *fptr, *ftmp;
@@ -371,20 +364,32 @@ int main(int argc, char *argv[])
 
 	OpenSSL_add_all_digests();
 
-	if(argc == 4){
+	if(argc == 5){
+		if(!strcmp(argv[1], "-b")){
+			batch_mode = true;
+		}else{
+			fprintf(stderr, "Error: Unknown option '%s'\n", argv[1]);
+		}
+		password_file_tmp = argv[2];
+		username = argv[3];
+		password_cmd = argv[4];
+	}else if(argc == 4){
 		if(!strcmp(argv[1], "-c")){
 			create_new = true;
 		}else if(!strcmp(argv[1], "-D")){
 			delete_user = true;
+		}else{
+			fprintf(stderr, "Error: Unknown option '%s'\n", argv[1]);
+			return 1;
 		}
-		password_file = argv[2];
+		password_file_tmp = argv[2];
 		username = argv[3];
 	}else if(argc == 3){
 		if(!strcmp(argv[1], "-U")){
 			do_update_file = true;
-			password_file = argv[2];
+			password_file_tmp = argv[2];
 		}else{
-			password_file = argv[1];
+			password_file_tmp = argv[1];
 			username = argv[2];
 		}
 	}else{
@@ -392,6 +397,8 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	snprintf(password_file, 1024, "%s", password_file_tmp);
+
 	if(create_new){
 		rc = get_password(password, 1024);
 		if(rc) return rc;
@@ -431,16 +438,21 @@ int main(int argc, char *argv[])
 		}else if(do_update_file){
 			rc = update_file(fptr, ftmp);
 		}else{
-			rc = get_password(password, 1024);
-			if(rc){
-				fclose(fptr);
-				fclose(ftmp);
-				unlink(backup_file);
-				free(backup_file);
-				return rc;
+			if(batch_mode){
+				/* Update password for individual user */
+				rc = update_pwuser(fptr, ftmp, username, password_cmd);
+			}else{
+				rc = get_password(password, 1024);
+				if(rc){
+					fclose(fptr);
+					fclose(ftmp);
+					unlink(backup_file);
+					free(backup_file);
+					return rc;
+				}
+				/* Update password for individual user */
+				rc = update_pwuser(fptr, ftmp, username, password);
 			}
-			/* Update password for individual user */
-			rc = update_pwuser(fptr, ftmp, username, password);
 		}
 		if(rc){
 			fclose(fptr);
