@@ -1,30 +1,17 @@
 /*
-Copyright (c) 2010-2013 Roger Light <roger@atchoo.org>
-All rights reserved.
+Copyright (c) 2010-2014 Roger Light <roger@atchoo.org>
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
-
-1. Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
-2. Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-3. Neither the name of mosquitto nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
-LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-POSSIBILITY OF SUCH DAMAGE.
+All rights reserved. This program and the accompanying materials
+are made available under the terms of the Eclipse Public License v1.0
+and Eclipse Distribution License v1.0 which accompany this distribution.
+ 
+The Eclipse Public License is available at
+   http://www.eclipse.org/legal/epl-v10.html
+and the Eclipse Distribution License is available at
+  http://www.eclipse.org/org/documents/edl-v10.php.
+ 
+Contributors:
+   Roger Light - initial implementation and documentation.
 */
 
 #include <config.h>
@@ -40,6 +27,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <time.h>
 
 #include <mosquitto_broker.h>
 #include <memory_mosq.h>
@@ -55,41 +43,21 @@ static int _db_restore_sub(struct mosquitto_db *db, const char *client_id, const
 static struct mosquitto *_db_find_or_add_context(struct mosquitto_db *db, const char *client_id, uint16_t last_mid)
 {
 	struct mosquitto *context;
-	struct mosquitto **tmp_contexts;
-	int i;
 
 	context = NULL;
-	for(i=0; i<db->context_count; i++){
-		if(db->contexts[i] && !strcmp(db->contexts[i]->id, client_id)){
-			context = db->contexts[i];
-			break;
-		}
-	}
+	HASH_FIND(hh_id, db->contexts_by_id, client_id, strlen(client_id), context);
 	if(!context){
-		context = mqtt3_context_init(-1);
+		context = mqtt3_context_init(db, -1);
 		if(!context) return NULL;
+		context->id = _mosquitto_strdup(client_id);
+		if(!context->id){
+			_mosquitto_free(context);
+			return NULL;
+		}
 
 		context->clean_session = false;
 
-		for(i=0; i<db->context_count; i++){
-			if(!db->contexts[i]){
-				db->contexts[i] = context;
-				break;
-			}
-		}
-		if(i==db->context_count){
-			db->context_count++;
-			tmp_contexts = _mosquitto_realloc(db->contexts, sizeof(struct mosquitto*)*db->context_count);
-			if(tmp_contexts){
-				db->contexts = tmp_contexts;
-				db->contexts[db->context_count-1] = context;
-			}else{
-				mqtt3_context_cleanup(db, context, true);
-				return NULL;
-			}
-		}
-		context->id = _mosquitto_strdup(client_id);
-		context->db_index = i;
+		HASH_ADD_KEYPTR(hh_id, db->contexts_by_id, context->id, strlen(context->id), context);
 	}
 	if(last_mid){
 		context->last_mid = last_mid;
@@ -171,7 +139,7 @@ static int mqtt3_db_message_store_write(struct mosquitto_db *db, FILE *db_fptr)
 
 	stored = db->msg_store;
 	while(stored){
-		if(!strncmp(stored->msg.topic, "$SYS", 4)){
+		if(!strncmp(stored->topic, "$SYS", 4)){
 			/* Don't save $SYS messages as retained otherwise they can give
 			 * misleading information when reloaded. They should still be saved
 			 * because a disconnected durable client may have them in their
@@ -182,8 +150,8 @@ static int mqtt3_db_message_store_write(struct mosquitto_db *db, FILE *db_fptr)
 		}
 		length = htonl(sizeof(dbid_t) + 2+strlen(stored->source_id) +
 				sizeof(uint16_t) + sizeof(uint16_t) +
-				2+strlen(stored->msg.topic) + sizeof(uint32_t) +
-				stored->msg.payloadlen + sizeof(uint8_t) + sizeof(uint8_t));
+				2+strlen(stored->topic) + sizeof(uint32_t) +
+				stored->payloadlen + sizeof(uint8_t) + sizeof(uint8_t));
 
 		i16temp = htons(DB_CHUNK_MSG_STORE);
 		write_e(db_fptr, &i16temp, sizeof(uint16_t));
@@ -202,30 +170,29 @@ static int mqtt3_db_message_store_write(struct mosquitto_db *db, FILE *db_fptr)
 		i16temp = htons(stored->source_mid);
 		write_e(db_fptr, &i16temp, sizeof(uint16_t));
 
-		i16temp = htons(stored->msg.mid);
+		i16temp = htons(stored->mid);
 		write_e(db_fptr, &i16temp, sizeof(uint16_t));
 
-		slen = strlen(stored->msg.topic);
+		slen = strlen(stored->topic);
 		i16temp = htons(slen);
 		write_e(db_fptr, &i16temp, sizeof(uint16_t));
-		write_e(db_fptr, stored->msg.topic, slen);
+		write_e(db_fptr, stored->topic, slen);
 
-		i8temp = (uint8_t )stored->msg.qos;
+		i8temp = (uint8_t )stored->qos;
 		write_e(db_fptr, &i8temp, sizeof(uint8_t));
 
 		if(force_no_retain == false){
-			i8temp = (uint8_t )stored->msg.retain;
+			i8temp = (uint8_t )stored->retain;
 		}else{
 			i8temp = 0;
 		}
 		write_e(db_fptr, &i8temp, sizeof(uint8_t));
 
-		i32temp = htonl(stored->msg.payloadlen);
+		i32temp = htonl(stored->payloadlen);
 		write_e(db_fptr, &i32temp, sizeof(uint32_t));
-		if(stored->msg.payloadlen){
-			write_e(db_fptr, stored->msg.payload, (unsigned int)stored->msg.payloadlen);
+		if(stored->payloadlen){
+			write_e(db_fptr, stored->payload, (unsigned int)stored->payloadlen);
 		}
-
 		stored = stored->next;
 	}
 
@@ -237,16 +204,15 @@ error:
 
 static int mqtt3_db_client_write(struct mosquitto_db *db, FILE *db_fptr)
 {
-	int i;
-	struct mosquitto *context;
+	struct mosquitto *context, *ctxt_tmp;
 	uint16_t i16temp, slen;
 	uint32_t length;
+	time_t disconnect_t;
 
 	assert(db);
 	assert(db_fptr);
 
-	for(i=0; i<db->context_count; i++){
-		context = db->contexts[i];
+	HASH_ITER(hh_id, db->contexts_by_id, context, ctxt_tmp){
 		if(context && context->clean_session == false){
 			length = htonl(2+strlen(context->id) + sizeof(uint16_t) + sizeof(time_t));
 
@@ -260,7 +226,12 @@ static int mqtt3_db_client_write(struct mosquitto_db *db, FILE *db_fptr)
 			write_e(db_fptr, context->id, slen);
 			i16temp = htons(context->last_mid);
 			write_e(db_fptr, &i16temp, sizeof(uint16_t));
-			write_e(db_fptr, &(context->disconnect_t), sizeof(time_t));
+			if(context->disconnect_t){
+				disconnect_t = context->disconnect_t;
+			}else{
+				disconnect_t = time(NULL);
+			}
+			write_e(db_fptr, &disconnect_t, sizeof(time_t));
 
 			if(mqtt3_db_client_messages_write(db, db_fptr, context)) return 1;
 		}
@@ -315,7 +286,7 @@ static int _db_subs_retain_write(struct mosquitto_db *db, FILE *db_fptr, struct
 		sub = sub->next;
 	}
 	if(node->retained){
-		if(strncmp(node->retained->msg.topic, "$SYS", 4)){
+		if(strncmp(node->retained->topic, "$SYS", 4)){
 			/* Don't save $SYS messages. */
 			length = htonl(sizeof(dbid_t));
 
@@ -353,7 +324,7 @@ static int mqtt3_db_subs_retain_write(struct mosquitto_db *db, FILE *db_fptr)
 	return MOSQ_ERR_SUCCESS;
 }
 
-int mqtt3_db_backup(struct mosquitto_db *db, bool cleanup, bool shutdown)
+int mqtt3_db_backup(struct mosquitto_db *db, bool shutdown)
 {
 	int rc = 0;
 	FILE *db_fptr = NULL;
@@ -369,17 +340,15 @@ int mqtt3_db_backup(struct mosquitto_db *db, bool cleanup, bool shutdown)
 
 	if(!db || !db->config || !db->config->persistence_filepath) return MOSQ_ERR_INVAL;
 	_mosquitto_log_printf(NULL, MOSQ_LOG_INFO, "Saving in-memory database to %s.", db->config->persistence_filepath);
-	if(cleanup){
-		mqtt3_db_store_clean(db);
-	}
 
 	len = strlen(db->config->persistence_filepath)+5;
-	outfile = _mosquitto_calloc(len+1, 1);
+	outfile = _mosquitto_malloc(len+1);
 	if(!outfile){
 		_mosquitto_log_printf(NULL, MOSQ_LOG_INFO, "Error saving in-memory database, out of memory.");
 		return MOSQ_ERR_NOMEM;
 	}
 	snprintf(outfile, len, "%s.new", db->config->persistence_filepath);
+	outfile[len] = '\0';
 	db_fptr = _mosquitto_fopen(outfile, "wb");
 	if(db_fptr == NULL){
 		_mosquitto_log_printf(NULL, MOSQ_LOG_INFO, "Error saving in-memory database, unable to open %s for writing.", outfile);
@@ -417,7 +386,9 @@ int mqtt3_db_backup(struct mosquitto_db *db, bool cleanup, bool shutdown)
 
 #ifdef WIN32
 	if(remove(db->config->persistence_filepath) != 0){
-		goto error;
+		if(errno != ENOENT){
+			goto error;
+		}
 	}
 #endif
 	if(rename(outfile, db->config->persistence_filepath) != 0){
@@ -437,37 +408,34 @@ error:
 static int _db_client_msg_restore(struct mosquitto_db *db, const char *client_id, uint16_t mid, uint8_t qos, uint8_t retain, uint8_t direction, uint8_t state, uint8_t dup, uint64_t store_id)
 {
 	struct mosquitto_client_msg *cmsg;
-	struct mosquitto_msg_store *store;
+	struct mosquitto_msg_store_load *load;
 	struct mosquitto *context;
 
-	cmsg = _mosquitto_calloc(1, sizeof(struct mosquitto_client_msg));
+	cmsg = _mosquitto_malloc(sizeof(struct mosquitto_client_msg));
 	if(!cmsg){
 		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		return MOSQ_ERR_NOMEM;
 	}
 
+	cmsg->next = NULL;
 	cmsg->store = NULL;
 	cmsg->mid = mid;
 	cmsg->qos = qos;
 	cmsg->retain = retain;
+	cmsg->timestamp = 0;
 	cmsg->direction = direction;
 	cmsg->state = state;
 	cmsg->dup = dup;
 
-	store = db->msg_store;
-	while(store){
-		if(store->db_id == store_id){
-			cmsg->store = store;
-			cmsg->store->ref_count++;
-			break;
-		}
-		store = store->next;
-	}
-	if(!cmsg->store){
+	HASH_FIND(hh, db->msg_store_load, &store_id, sizeof(dbid_t), load);
+	if(!load){
 		_mosquitto_free(cmsg);
 		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error restoring persistent database, message store corrupt.");
 		return 1;
 	}
+	cmsg->store = load->store;
+	cmsg->store->ref_count++;
+
 	context = _db_find_or_add_context(db, client_id, 0);
 	if(!context){
 		_mosquitto_free(cmsg);
@@ -479,7 +447,6 @@ static int _db_client_msg_restore(struct mosquitto_db *db, const char *client_id
 	}else{
 		context->msgs = cmsg;
 	}
-	cmsg->next = NULL;
 	context->last_msg = cmsg;
 
 	return MOSQ_ERR_SUCCESS;
@@ -492,7 +459,6 @@ static int _db_client_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 	int rc = 0;
 	struct mosquitto *context;
 	time_t disconnect_t;
-	struct _clientid_index_hash *new_cih;
 
 	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
@@ -501,19 +467,20 @@ static int _db_client_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 		fclose(db_fptr);
 		return 1;
 	}
-	client_id = _mosquitto_calloc(slen+1, sizeof(char));
+	client_id = _mosquitto_malloc(slen+1);
 	if(!client_id){
 		fclose(db_fptr);
 		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		return MOSQ_ERR_NOMEM;
 	}
 	read_e(db_fptr, client_id, slen);
+	client_id[slen] = '\0';
 
 	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	last_mid = ntohs(i16temp);
 
 	if(db_version == 2){
-		disconnect_t = mosquitto_time();
+		disconnect_t = time(NULL);
 	}else{
 		read_e(db_fptr, &disconnect_t, sizeof(time_t));
 	}
@@ -527,16 +494,6 @@ static int _db_client_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 
 	_mosquitto_free(client_id);
 
-	if(!rc){
-		new_cih = _mosquitto_malloc(sizeof(struct _clientid_index_hash));
-		if(!new_cih){
-			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
-			return MOSQ_ERR_NOMEM;
-		}
-		new_cih->id = context->id;
-		new_cih->db_context_index = context->db_index;
-		HASH_ADD_KEYPTR(hh, db->clientid_index_hash, context->id, strlen(context->id), new_cih);
-	}
 	return rc;
 error:
 	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
@@ -561,13 +518,14 @@ static int _db_client_msg_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 		fclose(db_fptr);
 		return 1;
 	}
-	client_id = _mosquitto_calloc(slen+1, sizeof(char));
+	client_id = _mosquitto_malloc(slen+1);
 	if(!client_id){
 		fclose(db_fptr);
 		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		return MOSQ_ERR_NOMEM;
 	}
 	read_e(db_fptr, client_id, slen);
+	client_id[slen] = '\0';
 
 	read_e(db_fptr, &i64temp, sizeof(dbid_t));
 	store_id = i64temp;
@@ -603,21 +561,31 @@ static int _db_msg_store_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 	char *topic = NULL;
 	int rc = 0;
 	struct mosquitto_msg_store *stored = NULL;
+	struct mosquitto_msg_store_load *load;
 	char err[256];
 
+	load = _mosquitto_malloc(sizeof(struct mosquitto_msg_store_load));
+	if(!load){
+		fclose(db_fptr);
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
+		return MOSQ_ERR_NOMEM;
+	}
+
 	read_e(db_fptr, &i64temp, sizeof(dbid_t));
 	store_id = i64temp;
 
 	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	if(slen){
-		source_id = _mosquitto_calloc(slen+1, sizeof(char));
+		source_id = _mosquitto_malloc(slen+1);
 		if(!source_id){
+			_mosquitto_free(load);
 			fclose(db_fptr);
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 			return MOSQ_ERR_NOMEM;
 		}
 		read_e(db_fptr, source_id, slen);
+		source_id[slen] = '\0';
 	}
 	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	source_mid = ntohs(i16temp);
@@ -628,15 +596,18 @@ static int _db_msg_store_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
 	if(slen){
-		topic = _mosquitto_calloc(slen+1, sizeof(char));
+		topic = _mosquitto_malloc(slen+1);
 		if(!topic){
+			_mosquitto_free(load);
 			fclose(db_fptr);
 			if(source_id) _mosquitto_free(source_id);
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 			return MOSQ_ERR_NOMEM;
 		}
 		read_e(db_fptr, topic, slen);
+		topic[slen] = '\0';
 	}else{
+		_mosquitto_free(load);
 		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Invalid msg_store chunk when restoring persistent database.");
 		fclose(db_fptr);
 		if(source_id) _mosquitto_free(source_id);
@@ -651,6 +622,7 @@ static int _db_msg_store_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 	if(payloadlen){
 		payload = _mosquitto_malloc(payloadlen);
 		if(!payload){
+			_mosquitto_free(load);
 			fclose(db_fptr);
 			if(source_id) _mosquitto_free(source_id);
 			_mosquitto_free(topic);
@@ -661,6 +633,12 @@ static int _db_msg_store_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 	}
 
 	rc = mqtt3_db_message_store(db, source_id, source_mid, topic, qos, payloadlen, payload, retain, &stored, store_id);
+
+	load->db_id = stored->db_id;
+	load->store = stored;
+
+	HASH_ADD(hh, db->msg_store_load, db_id, sizeof(dbid_t), load);
+
 	if(source_id) _mosquitto_free(source_id);
 	_mosquitto_free(topic);
 	_mosquitto_free(payload);
@@ -679,7 +657,7 @@ error:
 static int _db_retain_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 {
 	dbid_t i64temp, store_id;
-	struct mosquitto_msg_store *store;
+	struct mosquitto_msg_store_load *load;
 	char err[256];
 
 	if(fread(&i64temp, sizeof(dbid_t), 1, db_fptr) != 1){
@@ -689,13 +667,12 @@ static int _db_retain_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 		return 1;
 	}
 	store_id = i64temp;
-	store = db->msg_store;
-	while(store){
-		if(store->db_id == store_id){
-			mqtt3_db_messages_queue(db, NULL, store->msg.topic, store->msg.qos, store->msg.retain, store);
-			break;
-		}
-		store = store->next;
+	HASH_FIND(hh, db->msg_store_load, &store_id, sizeof(dbid_t), load);
+	if(load){
+		mqtt3_db_messages_queue(db, NULL, load->store->topic, load->store->qos, load->store->retain, &load->store);
+	}else{
+		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Corrupt database whilst restoring a retained message.");
+		return MOSQ_ERR_INVAL;
 	}
 	return MOSQ_ERR_SUCCESS;
 }
@@ -711,16 +688,18 @@ static int _db_sub_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 
 	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
-	client_id = _mosquitto_calloc(slen+1, sizeof(char));
+	client_id = _mosquitto_malloc(slen+1);
 	if(!client_id){
 		fclose(db_fptr);
 		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		return MOSQ_ERR_NOMEM;
 	}
 	read_e(db_fptr, client_id, slen);
+	client_id[slen] = '\0';
+
 	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
-	topic = _mosquitto_calloc(slen+1, sizeof(char));
+	topic = _mosquitto_malloc(slen+1);
 	if(!topic){
 		fclose(db_fptr);
 		_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
@@ -728,6 +707,8 @@ static int _db_sub_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 		return MOSQ_ERR_NOMEM;
 	}
 	read_e(db_fptr, topic, slen);
+	topic[slen] = '\0';
+
 	read_e(db_fptr, &qos, sizeof(uint8_t));
 	if(_db_restore_sub(db, client_id, topic, qos)){
 		rc = 1;
@@ -755,11 +736,14 @@ int mqtt3_db_restore(struct mosquitto_db *db)
 	uint8_t i8temp;
 	ssize_t rlen;
 	char err[256];
+	struct mosquitto_msg_store_load *load, *load_tmp;
 
 	assert(db);
 	assert(db->config);
 	assert(db->config->persistence_filepath);
 
+	db->msg_store_load = NULL;
+
 	fptr = _mosquitto_fopen(db->config->persistence_filepath, "rb");
 	if(fptr == NULL) return MOSQ_ERR_SUCCESS;
 	read_e(fptr, &header, 15);
@@ -833,6 +817,10 @@ int mqtt3_db_restore(struct mosquitto_db *db)
 
 	fclose(fptr);
 
+	HASH_ITER(hh, db->msg_store_load, load, load_tmp){
+		HASH_DELETE(hh, db->msg_store_load, load);
+		_mosquitto_free(load);
+	}
 	return rc;
 error:
 	strerror_r(errno, err, 256);
