@@ -1,30 +1,17 @@
 /*
-Copyright (c) 2011,2013 Roger Light <roger@atchoo.org>
-All rights reserved.
+Copyright (c) 2011-2014 Roger Light <roger@atchoo.org>
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
-
-1. Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
-2. Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-3. Neither the name of mosquitto nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
-LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-POSSIBILITY OF SUCH DAMAGE.
+All rights reserved. This program and the accompanying materials
+are made available under the terms of the Eclipse Public License v1.0
+and Eclipse Distribution License v1.0 which accompany this distribution.
+ 
+The Eclipse Public License is available at
+   http://www.eclipse.org/legal/epl-v10.html
+and the Eclipse Distribution License is available at
+  http://www.eclipse.org/org/documents/edl-v10.php.
+ 
+Contributors:
+   Roger Light - initial implementation and documentation.
 */
 
 #include <config.h>
@@ -46,6 +33,19 @@ typedef int (*FUNC_auth_plugin_acl_check)(void *, const char *, const char *, co
 typedef int (*FUNC_auth_plugin_unpwd_check)(void *, const char *, const char *);
 typedef int (*FUNC_auth_plugin_psk_key_get)(void *, const char *, const char *, char *, int);
 
+void LIB_ERROR(void)
+{
+#ifdef WIN32
+	char *buf;
+	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_STRING,
+			NULL, GetLastError(), LANG_NEUTRAL, &buf, 0, NULL);
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Load error: %s", buf);
+	LocalFree(buf);
+#else
+	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Load error: %s", dlerror());
+#endif
+}
+
 int mosquitto_security_module_init(struct mosquitto_db *db)
 {
 	void *lib;
@@ -57,6 +57,7 @@ int mosquitto_security_module_init(struct mosquitto_db *db)
 		if(!lib){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, 
 					"Error: Unable to load auth plugin \"%s\".", db->config->auth_plugin);
+			LIB_ERROR();
 			return 1;
 		}
 
@@ -64,6 +65,7 @@ int mosquitto_security_module_init(struct mosquitto_db *db)
 		if(!(plugin_version = (FUNC_auth_plugin_version)LIB_SYM(lib, "mosquitto_auth_plugin_version"))){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR,
 					"Error: Unable to load auth plugin function mosquitto_auth_plugin_version().");
+			LIB_ERROR();
 			LIB_CLOSE(lib);
 			return 1;
 		}
@@ -72,6 +74,7 @@ int mosquitto_security_module_init(struct mosquitto_db *db)
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR,
 					"Error: Incorrect auth plugin version (got %d, expected %d).",
 					version, MOSQ_AUTH_PLUGIN_VERSION);
+			LIB_ERROR();
 
 			LIB_CLOSE(lib);
 			return 1;
@@ -79,12 +82,14 @@ int mosquitto_security_module_init(struct mosquitto_db *db)
 		if(!(db->auth_plugin.plugin_init = (FUNC_auth_plugin_init)LIB_SYM(lib, "mosquitto_auth_plugin_init"))){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR,
 					"Error: Unable to load auth plugin function mosquitto_auth_plugin_init().");
+			LIB_ERROR();
 			LIB_CLOSE(lib);
 			return 1;
 		}
 		if(!(db->auth_plugin.plugin_cleanup = (FUNC_auth_plugin_cleanup)LIB_SYM(lib, "mosquitto_auth_plugin_cleanup"))){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR,
 					"Error: Unable to load auth plugin function mosquitto_auth_plugin_cleanup().");
+			LIB_ERROR();
 			LIB_CLOSE(lib);
 			return 1;
 		}
@@ -92,6 +97,7 @@ int mosquitto_security_module_init(struct mosquitto_db *db)
 		if(!(db->auth_plugin.security_init = (FUNC_auth_plugin_security_init)LIB_SYM(lib, "mosquitto_auth_security_init"))){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR,
 					"Error: Unable to load auth plugin function mosquitto_auth_security_init().");
+			LIB_ERROR();
 			LIB_CLOSE(lib);
 			return 1;
 		}
@@ -99,6 +105,7 @@ int mosquitto_security_module_init(struct mosquitto_db *db)
 		if(!(db->auth_plugin.security_cleanup = (FUNC_auth_plugin_security_cleanup)LIB_SYM(lib, "mosquitto_auth_security_cleanup"))){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR,
 					"Error: Unable to load auth plugin function mosquitto_auth_security_cleanup().");
+			LIB_ERROR();
 			LIB_CLOSE(lib);
 			return 1;
 		}
@@ -106,6 +113,7 @@ int mosquitto_security_module_init(struct mosquitto_db *db)
 		if(!(db->auth_plugin.acl_check = (FUNC_auth_plugin_acl_check)LIB_SYM(lib, "mosquitto_auth_acl_check"))){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR,
 					"Error: Unable to load auth plugin function mosquitto_auth_acl_check().");
+			LIB_ERROR();
 			LIB_CLOSE(lib);
 			return 1;
 		}
@@ -113,6 +121,7 @@ int mosquitto_security_module_init(struct mosquitto_db *db)
 		if(!(db->auth_plugin.unpwd_check = (FUNC_auth_plugin_unpwd_check)LIB_SYM(lib, "mosquitto_auth_unpwd_check"))){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR,
 					"Error: Unable to load auth plugin function mosquitto_auth_unpwd_check().");
+			LIB_ERROR();
 			LIB_CLOSE(lib);
 			return 1;
 		}
@@ -120,6 +129,7 @@ int mosquitto_security_module_init(struct mosquitto_db *db)
 		if(!(db->auth_plugin.psk_key_get = (FUNC_auth_plugin_psk_key_get)LIB_SYM(lib, "mosquitto_auth_psk_key_get"))){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR,
 					"Error: Unable to load auth plugin function mosquitto_auth_psk_key_get().");
+			LIB_ERROR();
 			LIB_CLOSE(lib);
 			return 1;
 		}
@@ -207,10 +217,23 @@ int mosquitto_security_cleanup(struct mosquitto_db *db, bool reload)
 
 int mosquitto_acl_check(struct mosquitto_db *db, struct mosquitto *context, const char *topic, int access)
 {
+	char *username;
+
+	if(!context->id){
+		return MOSQ_ERR_ACL_DENIED;
+	}
 	if(!db->auth_plugin.lib){
 		return mosquitto_acl_check_default(db, context, topic, access);
 	}else{
-		return db->auth_plugin.acl_check(db->auth_plugin.user_data, context->id, context->username, topic, access);
+#ifdef WITH_BRIDGE
+		if(context->bridge){
+			username = context->bridge->local_username;
+		}else
+#endif
+		{
+			username = context->username;
+		}
+		return db->auth_plugin.acl_check(db->auth_plugin.user_data, context->id, username, topic, access);
 	}
 }
 
