@@ -1,30 +1,17 @@
 /*
-Copyright (c) 2011-2013 Roger Light <roger@atchoo.org>
-All rights reserved.
+Copyright (c) 2011-2014 Roger Light <roger@atchoo.org>
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
-
-1. Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
-2. Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-3. Neither the name of mosquitto nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
-LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-POSSIBILITY OF SUCH DAMAGE.
+All rights reserved. This program and the accompanying materials
+are made available under the terms of the Eclipse Public License v1.0
+and Eclipse Distribution License v1.0 which accompany this distribution.
+ 
+The Eclipse Public License is available at
+   http://www.eclipse.org/legal/epl-v10.html
+and the Eclipse Distribution License is available at
+  http://www.eclipse.org/org/documents/edl-v10.php.
+ 
+Contributors:
+   Roger Light - initial implementation and documentation.
 */
 
 #include <config.h>
@@ -141,11 +128,16 @@ int _add_acl(struct mosquitto_db *db, const char *user, const char *topic, int a
 		acl_user->acl = NULL;
 	}
 
-	acl= _mosquitto_malloc(sizeof(struct _mosquitto_acl));
-	if(!acl) return MOSQ_ERR_NOMEM;
+	acl = _mosquitto_malloc(sizeof(struct _mosquitto_acl));
+	if(!acl){
+		_mosquitto_free(local_topic);
+		return MOSQ_ERR_NOMEM;
+	}
 	acl->access = access;
 	acl->topic = local_topic;
 	acl->next = NULL;
+	acl->ccount = 0;
+	acl->ucount = 0;
 
 	/* Add acl to user acl list */
 	if(acl_user->acl){
@@ -188,7 +180,10 @@ int _add_acl_pattern(struct mosquitto_db *db, const char *topic, int access)
 	}
 
 	acl = _mosquitto_malloc(sizeof(struct _mosquitto_acl));
-	if(!acl) return MOSQ_ERR_NOMEM;
+	if(!acl){
+		_mosquitto_free(local_topic);
+		return MOSQ_ERR_NOMEM;
+	}
 	acl->access = access;
 	acl->topic = local_topic;
 	acl->next = NULL;
@@ -306,6 +301,7 @@ int mosquitto_acl_check_default(struct mosquitto_db *db, struct mosquitto *conte
 		local_acl[len] = '\0';
 
 		mosquitto_topic_matches_sub(local_acl, topic, &result);
+		_mosquitto_free(local_acl);
 		if(result){
 			if(access & acl_root->access){
 				/* And access is allowed. */
@@ -313,8 +309,6 @@ int mosquitto_acl_check_default(struct mosquitto_db *db, struct mosquitto *conte
 			}
 		}
 
-		_mosquitto_free(local_acl);
-
 		acl_root = acl_root->next;
 	}
 
@@ -372,9 +366,13 @@ static int _aclfile_parse(struct mosquitto_db *db)
 					fclose(aclfile);
 					return MOSQ_ERR_INVAL;
 				}
-				token = strtok_r(NULL, " ", &saveptr);
+				token = strtok_r(NULL, "", &saveptr);
 				if(token){
 					topic = token;
+					/* Ignore duplicate spaces */
+					while(topic[0] == ' '){
+						topic++;
+					}
 				}else{
 					topic = access_s;
 					access_s = NULL;
@@ -384,8 +382,10 @@ static int _aclfile_parse(struct mosquitto_db *db)
 						access = MOSQ_ACL_READ;
 					}else if(!strcmp(access_s, "write")){
 						access = MOSQ_ACL_WRITE;
+					}else if(!strcmp(access_s, "readwrite")){
+						access = MOSQ_ACL_READ | MOSQ_ACL_WRITE;
 					}else{
-						_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Empty invalid topic access type in acl_file.");
+						_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Invalid topic access type \"%s\" in acl_file.", access_s);
 						if(user) _mosquitto_free(user);
 						fclose(aclfile);
 						return MOSQ_ERR_INVAL;
@@ -399,12 +399,17 @@ static int _aclfile_parse(struct mosquitto_db *db)
 					rc = _add_acl_pattern(db, topic, access);
 				}
 				if(rc){
+					if(user) _mosquitto_free(user);
 					fclose(aclfile);
 					return rc;
 				}
 			}else if(!strcmp(token, "user")){
-				token = strtok_r(NULL, " ", &saveptr);
+				token = strtok_r(NULL, "", &saveptr);
 				if(token){
+					/* Ignore duplicate spaces */
+					while(token[0] == ' '){
+						token++;
+					}
 					if(user) _mosquitto_free(user);
 					user = _mosquitto_strdup(token);
 					if(!user){
@@ -442,7 +447,7 @@ static void _free_acl(struct _mosquitto_acl *acl)
 
 static int _acl_cleanup(struct mosquitto_db *db, bool reload)
 {
-	int i;
+	struct mosquitto *context, *ctxt_tmp;
 	struct _mosquitto_acl_user *user_tail;
 
 	if(!db) return MOSQ_ERR_INVAL;
@@ -454,12 +459,8 @@ static int _acl_cleanup(struct mosquitto_db *db, bool reload)
 	 * is called if we are reloading the config. If this is not done, all 
 	 * access will be denied to currently connected clients.
 	 */
-	if(db->contexts){
-		for(i=0; i<db->context_count; i++){
-			if(db->contexts[i] && db->contexts[i]->acl_list){
-				db->contexts[i]->acl_list = NULL;
-			}
-		}
+	HASH_ITER(hh_id, db->contexts_by_id, context, ctxt_tmp){
+		context->acl_list = NULL;
 	}
 
 	while(db->acl_list){
@@ -703,49 +704,45 @@ static int _unpwd_cleanup(struct _mosquitto_unpwd **root, bool reload)
  */
 int mosquitto_security_apply_default(struct mosquitto_db *db)
 {
+	struct mosquitto *context, *ctxt_tmp;
 	struct _mosquitto_acl_user *acl_user_tail;
 	bool allow_anonymous;
-	int i;
 
 	if(!db) return MOSQ_ERR_INVAL;
 
 	allow_anonymous = db->config->allow_anonymous;
 	
-	if(db->contexts){
-		for(i=0; i<db->context_count; i++){
-			if(db->contexts[i]){
-				/* Check for anonymous clients when allow_anonymous is false */
-				if(!allow_anonymous && !db->contexts[i]->username){
-					db->contexts[i]->state = mosq_cs_disconnecting;
-					_mosquitto_socket_close(db->contexts[i]);
-					continue;
-				}
-				/* Check for connected clients that are no longer authorised */
-				if(mosquitto_unpwd_check_default(db, db->contexts[i]->username, db->contexts[i]->password) != MOSQ_ERR_SUCCESS){
-					db->contexts[i]->state = mosq_cs_disconnecting;
-					_mosquitto_socket_close(db->contexts[i]);
-					continue;
-				}
-				/* Check for ACLs and apply to user. */
-				if(db->acl_list){
-  					acl_user_tail = db->acl_list;
-					while(acl_user_tail){
-						if(acl_user_tail->username){
-							if(db->contexts[i]->username){
-								if(!strcmp(acl_user_tail->username, db->contexts[i]->username)){
-									db->contexts[i]->acl_list = acl_user_tail;
-									break;
-								}
-							}
-						}else{
-							if(!db->contexts[i]->username){
-								db->contexts[i]->acl_list = acl_user_tail;
-								break;
-							}
+	HASH_ITER(hh_id, db->contexts_by_id, context, ctxt_tmp){
+		/* Check for anonymous clients when allow_anonymous is false */
+		if(!allow_anonymous && !context->username){
+			context->state = mosq_cs_disconnecting;
+			do_disconnect(db, context);
+			continue;
+		}
+		/* Check for connected clients that are no longer authorised */
+		if(mosquitto_unpwd_check_default(db, context->username, context->password) != MOSQ_ERR_SUCCESS){
+			context->state = mosq_cs_disconnecting;
+			do_disconnect(db, context);
+			continue;
+		}
+		/* Check for ACLs and apply to user. */
+		if(db->acl_list){
+			acl_user_tail = db->acl_list;
+			while(acl_user_tail){
+				if(acl_user_tail->username){
+					if(context->username){
+						if(!strcmp(acl_user_tail->username, context->username)){
+							context->acl_list = acl_user_tail;
+							break;
 						}
-						acl_user_tail = acl_user_tail->next;
+					}
+				}else{
+					if(!context->username){
+						context->acl_list = acl_user_tail;
+						break;
 					}
 				}
+				acl_user_tail = acl_user_tail->next;
 			}
 		}
 	}
@@ -803,12 +800,12 @@ int _base64_decode(char *in, unsigned char **decoded, unsigned int *decoded_len)
 	BIO_write(bmem, in, strlen(in));
 
 	if(BIO_flush(bmem) != 1){
-		BIO_free_all(bmem);
+		BIO_free_all(b64);
 		return 1;
 	}
 	*decoded = calloc(strlen(in), 1);
 	*decoded_len =  BIO_read(b64, *decoded, strlen(in));
-	BIO_free_all(bmem);
+	BIO_free_all(b64);
 
 	return 0;
 }
