@@ -1,30 +1,17 @@
 /*
-Copyright (c) 2009-2013 Roger Light <roger@atchoo.org>
-All rights reserved.
+Copyright (c) 2009-2014 Roger Light <roger@atchoo.org>
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
-
-1. Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
-2. Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-3. Neither the name of mosquitto nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
-LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-POSSIBILITY OF SUCH DAMAGE.
+All rights reserved. This program and the accompanying materials
+are made available under the terms of the Eclipse Public License v1.0
+and Eclipse Distribution License v1.0 which accompany this distribution.
+ 
+The Eclipse Public License is available at
+   http://www.eclipse.org/legal/epl-v10.html
+and the Eclipse Distribution License is available at
+  http://www.eclipse.org/org/documents/edl-v10.php.
+ 
+Contributors:
+   Roger Light - initial implementation and documentation.
 */
 
 #ifdef WITH_SYS_TREE
@@ -58,35 +45,38 @@ static void _sys_update_clients(struct mosquitto_db *db, char *buf)
 	static unsigned int client_count = -1;
 	static int clients_expired = -1;
 	static unsigned int client_max = -1;
-	static unsigned int inactive_count = -1;
-	static unsigned int active_count = -1;
-	unsigned int value;
-	unsigned int inactive;
-	unsigned int active;
+	static unsigned int disconnected_count = -1;
+	static unsigned int connected_count = -1;
 
-	if(!mqtt3_db_client_count(db, &value, &inactive)){
-		if(client_count != value){
-			client_count = value;
-			snprintf(buf, BUFLEN, "%d", client_count);
-			mqtt3_db_messages_easy_queue(db, NULL, "$SYS/broker/clients/total", 2, strlen(buf), buf, 1);
-		}
-		if(inactive_count != inactive){
-			inactive_count = inactive;
-			snprintf(buf, BUFLEN, "%d", inactive_count);
-			mqtt3_db_messages_easy_queue(db, NULL, "$SYS/broker/clients/inactive", 2, strlen(buf), buf, 1);
-		}
-		active = client_count - inactive;
-		if(active_count != active){
-			active_count = active;
-			snprintf(buf, BUFLEN, "%d", active_count);
-			mqtt3_db_messages_easy_queue(db, NULL, "$SYS/broker/clients/active", 2, strlen(buf), buf, 1);
-		}
-		if(value != client_max){
-			client_max = value;
+	unsigned int count_total, count_by_sock;
+
+	count_total = HASH_CNT(hh_id, db->contexts_by_id);
+	count_by_sock = HASH_CNT(hh_sock, db->contexts_by_sock);
+
+	if(client_count != count_total){
+		client_count = count_total;
+		snprintf(buf, BUFLEN, "%d", client_count);
+		mqtt3_db_messages_easy_queue(db, NULL, "$SYS/broker/clients/total", 2, strlen(buf), buf, 1);
+
+		if(client_count > client_max){
+			client_max = client_count;
 			snprintf(buf, BUFLEN, "%d", client_max);
 			mqtt3_db_messages_easy_queue(db, NULL, "$SYS/broker/clients/maximum", 2, strlen(buf), buf, 1);
 		}
 	}
+
+	if(disconnected_count != count_total-count_by_sock){
+		disconnected_count = count_total-count_by_sock;
+		snprintf(buf, BUFLEN, "%d", disconnected_count);
+		mqtt3_db_messages_easy_queue(db, NULL, "$SYS/broker/clients/inactive", 2, strlen(buf), buf, 1);
+		mqtt3_db_messages_easy_queue(db, NULL, "$SYS/broker/clients/disconnected", 2, strlen(buf), buf, 1);
+	}
+	if(connected_count != count_by_sock){
+		connected_count = count_by_sock;
+		snprintf(buf, BUFLEN, "%d", connected_count);
+		mqtt3_db_messages_easy_queue(db, NULL, "$SYS/broker/clients/active", 2, strlen(buf), buf, 1);
+		mqtt3_db_messages_easy_queue(db, NULL, "$SYS/broker/clients/connected", 2, strlen(buf), buf, 1);
+	}
 	if(g_clients_expired != clients_expired){
 		clients_expired = g_clients_expired;
 		snprintf(buf, BUFLEN, "%d", clients_expired);
