@@ -19,18 +19,13 @@ keepalive = 10
 connect_packet = mosq_test.gen_connect("", keepalive=keepalive)
 connack_packet = mosq_test.gen_connack(rc=2)
 
-broker = subprocess.Popen(['../../src/mosquitto', '-p', '1888'], stderr=subprocess.PIPE)
+cmd = ['../../src/mosquitto', '-p', '1888']
+broker = mosq_test.start_broker(filename=os.path.basename(__file__), cmd=cmd)
 
 try:
-    time.sleep(0.5)
-
-    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    sock.connect(("localhost", 1888))
-    sock.send(connect_packet)
-    if mosq_test.expect_packet(sock, "connack", connack_packet):
-        rc = 0
-
+    sock = mosq_test.do_client_connect(connect_packet, connack_packet)
     sock.close()
+    rc = 0
 finally:
     broker.terminate()
     broker.wait()
