@@ -18,13 +18,11 @@ import mosq_test
 rc = 1
 keepalive = 10
 connect_packet = mosq_test.gen_connect("connect-uname-test", keepalive=keepalive, username="user")
-connack_packet = mosq_test.gen_connack(rc=4)
+connack_packet = mosq_test.gen_connack(rc=5)
 
-broker = subprocess.Popen(['../../src/mosquitto', '-c', '01-connect-uname-no-password-denied.conf'], stderr=subprocess.PIPE)
+broker = mosq_test.start_broker(filename=os.path.basename(__file__))
 
 try:
-    time.sleep(0.5)
-
     sock = mosq_test.do_client_connect(connect_packet, connack_packet)
     sock.close()
     rc = 0
