@@ -35,11 +35,10 @@ pubrec_packet2 = mosq_test.gen_pubrec(mid)
 pubrel_packet2 = mosq_test.gen_pubrel(mid)
 pubcomp_packet2 = mosq_test.gen_pubcomp(mid)
 
-broker = subprocess.Popen(['../../src/mosquitto', '-p', '1888'], stderr=subprocess.PIPE)
+cmd = ['../../src/mosquitto', '-p', '1888']
+broker = mosq_test.start_broker(filename=os.path.basename(__file__), cmd=cmd)
 
 try:
-    time.sleep(0.5)
-
     sock = mosq_test.do_client_connect(connect_packet, connack_packet, timeout=20)
     sock.send(subscribe_packet)
 
