@@ -23,20 +23,15 @@ connack_packet = mosq_test.gen_connack(rc=0)
 unsubscribe_packet = mosq_test.gen_unsubscribe(mid, "qos2/test")
 unsuback_packet = mosq_test.gen_unsuback(mid)
 
-broker = subprocess.Popen(['../../src/mosquitto', '-p', '1888'], stderr=subprocess.PIPE)
+cmd = ['../../src/mosquitto', '-p', '1888']
+broker = mosq_test.start_broker(filename=os.path.basename(__file__), cmd=cmd)
 
 try:
-    time.sleep(0.5)
+    sock = mosq_test.do_client_connect(connect_packet, connack_packet)
+    sock.send(unsubscribe_packet)
 
-    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    sock.connect(("localhost", 1888))
-    sock.send(connect_packet)
-
-    if mosq_test.expect_packet(sock, "connack", connack_packet):
-        sock.send(unsubscribe_packet)
-
-        if mosq_test.expect_packet(sock, "unsuback", unsuback_packet):
-            rc = 0
+    if mosq_test.expect_packet(sock, "unsuback", unsuback_packet):
+        rc = 0
 
     sock.close()
 finally:
