@@ -19,13 +19,9 @@ connack_packet = mosq_test.gen_connack(rc=0)
 
 publish_packet = mosq_test.gen_publish(sys.argv[1], qos=0, retain=True, payload="message")
 
-sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-sock.connect(("localhost", 1888))
-sock.send(connect_packet)
-if mosq_test.expect_packet(sock, "helper connack", connack_packet):
-    sock.send(publish_packet)
-    rc = 0
-
+sock = mosq_test.do_client_connect(connect_packet, connack_packet, connack_error="helper connack")
+sock.send(publish_packet)
+rc = 0
 sock.close()
     
 exit(rc)
