@@ -23,15 +23,11 @@ mid = 128
 publish_packet = mosq_test.gen_publish("qos1/timeout/test", qos=1, mid=mid, payload="timeout-message")
 puback_packet = mosq_test.gen_puback(mid)
 
-sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-sock.connect(("localhost", 1888))
-sock.send(connect_packet)
+sock = mosq_test.do_client_connect(connect_packet, connack_packet, connack_error="helper connack")
+sock.send(publish_packet)
 
-if mosq_test.expect_packet(sock, "helper connack", connack_packet):
-    sock.send(publish_packet)
-
-    if mosq_test.expect_packet(sock, "helper puback", puback_packet):
-        rc = 0
+if mosq_test.expect_packet(sock, "helper puback", puback_packet):
+    rc = 0
 
 sock.close()
     
