@@ -30,11 +30,10 @@ mid_unsub = 593
 unsubscribe_packet = mosq_test.gen_unsubscribe(mid_unsub, "retain/clear/test")
 unsuback_packet = mosq_test.gen_unsuback(mid_unsub)
 
-broker = subprocess.Popen(['../../src/mosquitto', '-p', '1888'], stderr=subprocess.PIPE)
+cmd = ['../../src/mosquitto', '-p', '1888']
+broker = mosq_test.start_broker(filename=os.path.basename(__file__), cmd=cmd)
 
 try:
-    time.sleep(0.5)
-
     sock = mosq_test.do_client_connect(connect_packet, connack_packet, timeout=4)
     # Send retained message
     sock.send(publish_packet)
