@@ -24,11 +24,10 @@ publish_packet = mosq_test.gen_publish("retain/qos0/test", qos=0, payload="retai
 subscribe_packet = mosq_test.gen_subscribe(mid, "retain/qos0/test", 0)
 suback_packet = mosq_test.gen_suback(mid, 0)
 
-broker = subprocess.Popen(['../../src/mosquitto', '-p', '1888'], stderr=subprocess.PIPE)
+cmd = ['../../src/mosquitto', '-p', '1888']
+broker = mosq_test.start_broker(filename=os.path.basename(__file__), cmd=cmd)
 
 try:
-    time.sleep(0.5)
-
     sock = mosq_test.do_client_connect(connect_packet, connack_packet)
     sock.send(publish_packet)
     sock.send(subscribe_packet)
