@@ -24,11 +24,9 @@ keepalive = 10
 connect_packet = mosq_test.gen_connect("connect-identity-test", keepalive=keepalive)
 connack_packet = mosq_test.gen_connack(rc=0)
 
-broker = subprocess.Popen(['../../src/mosquitto', '-c', '08-ssl-connect-identity.conf'], stderr=subprocess.PIPE)
+broker = mosq_test.start_broker(filename=os.path.basename(__file__), port=1889)
 
 try:
-    time.sleep(0.5)
-
     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     ssock = ssl.wrap_socket(sock, ca_certs="../ssl/test-root-ca.crt", certfile="../ssl/client.crt", keyfile="../ssl/client.key", cert_reqs=ssl.CERT_REQUIRED)
     ssock.settimeout(20)
@@ -40,6 +38,7 @@ try:
 
     ssock.close()
 finally:
+    time.sleep(0.5)
     broker.terminate()
     broker.wait()
     if rc:
