@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 # Client connects without a certificate to a server that has use_identity_as_username=true. Should be rejected.
 import subprocess
@@ -24,14 +24,12 @@ keepalive = 10
 connect_packet = mosq_test.gen_connect("connect-no-identity-test", keepalive=keepalive)
 connack_packet = mosq_test.gen_connack(rc=4)
 
-broker = subprocess.Popen(['../../src/mosquitto', '-c', '08-ssl-connect-no-identity.conf'], stderr=subprocess.PIPE)
+broker = mosq_test.start_broker(filename=os.path.basename(__file__), port=1889)
 
 try:
-    time.sleep(0.5)
-
     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    ssock = ssl.wrap_socket(sock, ca_certs="../ssl/test-ca.crt", cert_reqs=ssl.CERT_REQUIRED)
-    ssock.settimeout(5)
+    ssock = ssl.wrap_socket(sock, ca_certs="../ssl/test-root-ca.crt", cert_reqs=ssl.CERT_REQUIRED)
+    ssock.settimeout(20)
     ssock.connect(("localhost", 1888))
     ssock.send(connect_packet)
 
@@ -40,6 +38,7 @@ try:
 
     ssock.close()
 finally:
+    time.sleep(2)
     broker.terminate()
     broker.wait()
     if rc:
