@@ -24,25 +24,19 @@ suback_packet = mosq_test.gen_suback(mid, 0)
 
 publish_packet = mosq_test.gen_publish("mount/test", qos=0, payload="mount point")
 
-broker = subprocess.Popen(['../../src/mosquitto', '-c', '10-listener-mount-point.conf'], stderr=subprocess.PIPE)
+broker = mosq_test.start_broker(filename=os.path.basename(__file__))
 
 try:
-    time.sleep(0.5)
+    sock = mosq_test.do_client_connect(connect_packet, connack_packet, timeout=20)
+    sock.send(subscribe_packet)
 
-    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    sock.settimeout(20)
-    sock.connect(("localhost", 1888))
-    sock.send(connect_packet)
-    if mosq_test.expect_packet(sock, "connack", connack_packet):
-        sock.send(subscribe_packet)
+    if mosq_test.expect_packet(sock, "suback", suback_packet):
+        pub = subprocess.Popen(['./10-listener-mount-point-helper.py'], stdout=subprocess.PIPE)
+        pub.wait()
+        # Should have now received a publish command
 
-        if mosq_test.expect_packet(sock, "suback", suback_packet):
-            pub = subprocess.Popen(['./10-listener-mount-point-helper.py'], stdout=subprocess.PIPE)
-            pub.wait()
-            # Should have now received a publish command
-
-            if mosq_test.expect_packet(sock, "publish", publish_packet):
-                rc = 0
+        if mosq_test.expect_packet(sock, "publish", publish_packet):
+            rc = 0
 
     sock.close()
 finally:
