@@ -42,18 +42,26 @@ int main(int argc, char *argv[])
 	mosq = mosquitto_new("08-tls-psk-pub", true, NULL);
 	mosquitto_tls_opts_set(mosq, 1, "tlsv1", NULL);
 	rc = mosquitto_tls_psk_set(mosq, "deadbeef", "psk-id", NULL);
-	if(rc) return rc;
+	if(rc){
+		mosquitto_destroy(mosq);
+		return rc;
+	}
 	mosquitto_connect_callback_set(mosq, on_connect);
 	mosquitto_disconnect_callback_set(mosq, on_disconnect);
 	mosquitto_publish_callback_set(mosq, on_publish);
 
 	rc = mosquitto_connect(mosq, "localhost", 1888, 60);
-	if(rc) return rc;
+	if(rc){
+		mosquitto_destroy(mosq);
+		return rc;
+	}
 
 	while(run == -1){
 		mosquitto_loop(mosq, -1, 1);
 	}
 
+	mosquitto_destroy(mosq);
+
 	mosquitto_lib_cleanup();
 	return run;
 }
