@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 # Test whether a client sends a correct UNSUBSCRIBE packet.
 
@@ -41,7 +41,7 @@ try:
 except KeyError:
     pp = ''
 env['PYTHONPATH'] = '../../lib/python:'+pp
-client = subprocess.Popen(client_args, env=env)
+client = mosq_test.start_client(filename=sys.argv[1].replace('/', '-'), cmd=client_args, env=env)
 
 try:
     (conn, address) = sock.accept()
