@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 # Test whether a client sends a correct PUBLISH to a topic with QoS 1 and responds to a delay.
 
@@ -11,7 +11,7 @@
 # "pub/qos2/test", payload "message" and QoS=2.
 # The test will not respond to the first PUBLISH message, so the client must
 # resend the PUBLISH message with dup=1. Note that to keep test durations low, a
-# message retry timeout of less than 5 seconds is required for this test.
+# message retry timeout of less than 10 seconds is required for this test.
 # On receiving the second PUBLISH message, the test will send the correct
 # PUBREC response. On receiving the correct PUBREC response, the client should
 # send a PUBREL message.
@@ -46,7 +46,6 @@ publish_packet = mosq_test.gen_publish("pub/qos2/test", qos=2, mid=mid, payload=
 publish_dup_packet = mosq_test.gen_publish("pub/qos2/test", qos=2, mid=mid, payload="message", dup=True)
 pubrec_packet = mosq_test.gen_pubrec(mid)
 pubrel_packet = mosq_test.gen_pubrel(mid)
-pubrel_dup_packet = mosq_test.gen_pubrel(mid, dup=True)
 pubcomp_packet = mosq_test.gen_pubcomp(mid)
 
 sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -63,11 +62,11 @@ try:
 except KeyError:
     pp = ''
 env['PYTHONPATH'] = '../../lib/python:'+pp
-client = subprocess.Popen(client_args, env=env)
+client = mosq_test.start_client(filename=sys.argv[1].replace('/', '-'), cmd=client_args, env=env)
 
 try:
     (conn, address) = sock.accept()
-    conn.settimeout(5)
+    conn.settimeout(10)
 
     if mosq_test.expect_packet(conn, "connect", connect_packet):
         conn.send(connack_packet)
@@ -79,7 +78,7 @@ try:
                 conn.send(pubrec_packet)
                 
                 if mosq_test.expect_packet(conn, "pubrel", pubrel_packet):
-                    if mosq_test.expect_packet(conn, "dup pubrel", pubrel_dup_packet):
+                    if mosq_test.expect_packet(conn, "dup pubrel", pubrel_packet):
                         conn.send(pubcomp_packet)
 
                         if mosq_test.expect_packet(conn, "disconnect", disconnect_packet):
