@@ -30,7 +30,7 @@ except KeyError:
     pp = ''
 env['PYTHONPATH'] = '../../lib/python:'+pp
 
-client = subprocess.Popen(client_args, env=env)
+client = mosq_test.start_client(filename=sys.argv[1].replace('/', '-'), cmd=client_args, env=env)
 client.wait()
 
 rc = client.returncode
