@@ -10,6 +10,7 @@ import time
 cmd_subfolder = os.path.realpath(os.path.abspath(os.path.join(os.path.split(inspect.getfile( inspect.currentframe() ))[0],"..")))
 if cmd_subfolder not in sys.path:
     sys.path.insert(0, cmd_subfolder)
+import mosq_test
 
 rc = 1
 
@@ -22,7 +23,7 @@ except KeyError:
     pp = ''
 env['PYTHONPATH'] = '../../lib/python:'+pp
 
-client = subprocess.Popen(client_args, env=env)
+client = mosq_test.start_client(filename=sys.argv[1].replace('/', '-'), cmd=client_args, env=env)
 client.wait()
 if client.returncode:
      (stdo, stde) = client.communicate()
