@@ -28,7 +28,8 @@ int main(int argc, char *argv[])
 	mosquitto_lib_init();
 
 	mosq = mosquitto_new("08-ssl-connect-crt-auth", true, NULL);
-	mosquitto_tls_set(mosq, "../ssl/test-ca.crt", NULL, "../ssl/client.crt", "../ssl/client.key", NULL);
+	mosquitto_tls_opts_set(mosq, 1, "tlsv1", NULL);
+	mosquitto_tls_set(mosq, "../ssl/test-root-ca.crt", "../ssl/certs", "../ssl/client.crt", "../ssl/client.key", NULL);
 	mosquitto_connect_callback_set(mosq, on_connect);
 	mosquitto_disconnect_callback_set(mosq, on_disconnect);
 
