@@ -38,7 +38,9 @@ int main(int argc, char *argv[])
 
 	mosq = new mosquittopp_test("08-ssl-connect-no-auth");
 
-	mosq->tls_set("../ssl/test-ca.crt");
+	mosq->tls_opts_set(1, "tlsv1", NULL);
+	//mosq->tls_set("../ssl/test-root-ca.crt");
+	mosq->tls_set("../ssl/all-ca.crt");
 	mosq->connect("localhost", 1888, 60);
 
 	while(run == -1){
